@@ -1,6 +1,6 @@
 // -*- sateh-c -*- 
 // File    : pdnsbackend.cc
-// Version : $Id: pipebackend.cc 1288 2008-11-16 09:10:08Z ahu $ 
+// Version : $Id: pipebackend.cc 2239 2011-07-19 08:28:13Z ahu $ 
 //
 
 #include <string>
@@ -10,7 +10,7 @@
 #include <sstream>
 #include "coprocess.hh"
 
-using namespace std;
+#include "pdns/namespaces.hh"
 
 #include <pdns/dns.hh>
 #include <pdns/dnsbackend.hh>
@@ -23,9 +23,6 @@ using namespace std;
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <boost/lexical_cast.hpp>
-
-using namespace boost;
-
 #include "pipebackend.hh"
 
 static const char *kBackendId = "[PIPEBackend]";
@@ -116,19 +113,22 @@ void PipeBackend::lookup(const QType &qtype,const string &qname, DNSPacket *pkt_
          ostringstream query;
          string localIP="0.0.0.0";
          string remoteIP="0.0.0.0";
-         
+         Netmask realRemote;
          if (pkt_p) {
             localIP=pkt_p->getLocal();
-	    remoteIP=pkt_p->getRemote();
+            realRemote = pkt_p->getRealRemote();
+            remoteIP = pkt_p->getRemote();
          }
-
+         int abiVersion = ::arg().asNum("pipebackend-abi-version");
          // pipebackend-abi-version = 1
          // type    qname           qclass  qtype   id      remote-ip-address
          query<<"Q\t"<<qname<<"\tIN\t"<<qtype.getName()<<"\t"<<zoneId<<"\t"<<remoteIP;
 
          // add the local-ip-address if pipebackend-abi-version is set to 2
-         if (::arg().asNum("pipebackend-abi-version") == 2) 
+         if (abiVersion >= 2)
             query<<"\t"<<localIP;
+         if(abiVersion >= 3)
+           query <<"\t"<<realRemote.toString(); 
 
          if(::arg().mustDo("query-logging"))
             L<<Logger::Error<<"Query: '"<<query.str()<<"'"<<endl;
@@ -190,8 +190,13 @@ bool PipeBackend::get(DNSResourceRecord &r)
 
    // The answer format:
    // DATA    qname           qclass  qtype   ttl     id      content 
-
+   int abiVersion = ::arg().asNum("pipebackend-abi-version");
+   unsigned int extraFields = 0;
+   if(abiVersion == 3)
+     extraFields = 2;
+     
    for(;;) {
+     
       d_coproc->receive(line);
       vector<string>parts;
       stringtok(parts,line,"\t");
@@ -210,34 +215,43 @@ bool PipeBackend::get(DNSResourceRecord &r)
          continue;
       }
       else if(parts[0]=="DATA") { // yay
-         if(parts.size()<7) {
+         if(parts.size() < 7 + extraFields) {
             L<<Logger::Error<<kBackendId<<" coprocess returned incomplete or empty line in data section for query for "<<d_qname<<endl;
             throw AhuException("Format error communicating with coprocess in data section");
             // now what?
          }
-         r.qname=parts[1];
-         r.qtype=parts[3];
-         r.ttl=atoi(parts[4].c_str());
-         r.domain_id=atoi(parts[5].c_str());
+         
+         if(abiVersion == 3) {
+           r.scopeMask = atoi(parts[1].c_str());
+           r.auth = atoi(parts[2].c_str());
+         } else {
+           r.scopeMask = 0;
+           r.auth = 1;
+         }
+         r.qname=parts[1+extraFields];
+         r.qtype=parts[3+extraFields];
+         r.ttl=atoi(parts[4+extraFields].c_str());
+         r.domain_id=atoi(parts[5+extraFields].c_str());
+         
+         
  
-	 if(parts[3]!="MX") {
-	   r.content.clear();
-	   for(int n=6; n < parts.size(); ++n) {
-	     if(n!=6)
-	       r.content.append(1,' ');
-	     r.content.append(parts[n]);
-	   }
-	 }
-	 else {
-	   if(parts.size()<8) {
-            L<<Logger::Error<<kBackendId<<" coprocess returned incomplete MX line in data section for query for "<<d_qname<<endl;
-            throw AhuException("Format error communicating with coprocess in data section of MX record");
-	   }
-	   
-	   r.priority=atoi(parts[6].c_str());
-	   r.content=parts[7];
-
-	 }
+         if(r.qtype.getCode() != QType::MX && r.qtype.getCode() != QType::SRV) {
+           r.content.clear();
+           for(unsigned int n= 6 + extraFields; n < parts.size(); ++n) {
+             if(n!=6)
+               r.content.append(1,' ');
+             r.content.append(parts[n]);
+           }
+         }
+         else {
+           if(parts.size()< 8 + extraFields) {
+            L<<Logger::Error<<kBackendId<<" coprocess returned incomplete MX/SRV line in data section for query for "<<d_qname<<endl;
+            throw AhuException("Format error communicating with coprocess in data section of MX/SRV record");
+           }
+           
+           r.priority=atoi(parts[6+extraFields].c_str());
+           r.content=parts[7+extraFields];
+         }
          break;
       }
       else
