@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2005 - 2008  PowerDNS.COM BV
+    Copyright (C) 2005 - 2009  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as 
@@ -18,6 +18,7 @@
 
 #include "utility.hh"
 #include "dnsrecords.hh"
+#include <boost/foreach.hpp>
 
 boilerplate_conv(A, ns_t_a, conv.xfrIP(d_ip));
 
@@ -90,93 +91,6 @@ private:
 };
 
 
-void NSECRecordContent::report(void)
-{
-  regist(1, 47, &make, &make, "NSEC");
-}
-
-DNSRecordContent* NSECRecordContent::make(const string& content)
-{
-  return new NSECRecordContent(content);
-}
-
-NSECRecordContent::NSECRecordContent(const string& content, const string& zone) : DNSRecordContent(47)
-{
-  RecordTextReader rtr(content, zone);
-  rtr.xfrLabel(d_next);
-
-  while(!rtr.eof()) {
-    uint16_t type;
-    rtr.xfrType(type);
-    d_set.insert(type);
-  }
-}
-
-void NSECRecordContent::toPacket(DNSPacketWriter& pw)
-{
-  pw.xfrLabel(d_next);
-
-  uint8_t res[34];
-  memset(res, 0, sizeof(res));
-
-  set<uint16_t>::const_iterator i;
-  for(i=d_set.begin(); i != d_set.end() && *i<255; ++i){
-    res[2+*i/8] |= 1 << (7-(*i%8));
-  }
-  int len=0;
-  if(!d_set.empty()) 
-    len=1+*--i/8;
-
-  res[1]=len;
-
-  string tmp;
-  tmp.assign(res, res+len+2);
-  pw.xfrBlob(tmp);
-}
-
-NSECRecordContent::DNSRecordContent* NSECRecordContent::make(const DNSRecord &dr, PacketReader& pr) 
-{
-  NSECRecordContent* ret=new NSECRecordContent();
-  pr.xfrLabel(ret->d_next);
-  string bitmap;
-  pr.xfrBlob(bitmap);
-  
-  // 00 06 20 00 00 00 00 03  -> NS RRSIG NSEC  ( 2, 46, 47 ) counts from left
-  
-  if(bitmap.size() < 2)
-    throw MOADNSException("NSEC record with impossibly small bitmap");
-  
-  if(bitmap[0])
-    throw MOADNSException("Can't deal with NSEC mappings > 255 yet");
-  
-  unsigned int len=bitmap[1];
-  if(bitmap.size()!=2+len)
-    throw MOADNSException("Can't deal with multi-part NSEC mappings yet");
-  
-  for(unsigned int n=0 ; n < len ; ++n) {
-    uint8_t val=bitmap[2+n];
-    for(int bit = 0; bit < 8 ; ++bit , val>>=1)
-      if(val & 1) {
-	ret->d_set.insert((7-bit) + 8*(n));
-      }
-  }
-  
-  return ret;
-}
-
-string NSECRecordContent::getZoneRepresentation() const
-{
-  string ret;
-  RecordTextWriter rtw(ret);
-  rtw.xfrLabel(d_next);
-  
-  for(set<uint16_t>::const_iterator i=d_set.begin(); i!=d_set.end(); ++i) {
-    ret+=" ";
-    ret+=NumberToType(*i);
-  }
-  
-  return ret;
-}
 
 boilerplate_conv(NS, ns_t_ns, conv.xfrLabel(d_content, true));
 boilerplate_conv(PTR, ns_t_ptr, conv.xfrLabel(d_content, true));
@@ -187,22 +101,22 @@ boilerplate_conv(SPF, 99, conv.xfrText(d_text, true));
 boilerplate_conv(HINFO, ns_t_hinfo,  conv.xfrText(d_cpu);   conv.xfrText(d_host));
 
 boilerplate_conv(RP, ns_t_rp,
-		 conv.xfrLabel(d_mbox);   
-		 conv.xfrLabel(d_info)
-		 );
+        	 conv.xfrLabel(d_mbox);   
+        	 conv.xfrLabel(d_info)
+        	 );
 
 
 boilerplate_conv(OPT, ns_t_opt, 
-		   conv.xfrBlob(d_data)
-		 );
+        	   conv.xfrBlob(d_data)
+        	 );
 
 void OPTRecordContent::getData(vector<pair<uint16_t, string> >& options)
 {
   string::size_type pos=0;
   uint16_t code, len;
   while(d_data.size() >= 4 + pos) {
-    code = 0xff * d_data[pos] + d_data[pos+1];
-    len = 0xff * d_data[pos+2] + d_data[pos+3];
+    code = 256 * (unsigned char)d_data[pos] + (unsigned char)d_data[pos+1];
+    len = 256 * (unsigned char)d_data[pos+2] + (unsigned char)d_data[pos+3];
     pos+=4;
 
     if(pos + len > d_data.size())
@@ -215,57 +129,57 @@ void OPTRecordContent::getData(vector<pair<uint16_t, string> >& options)
 }
 
 boilerplate_conv(TSIG, ns_t_tsig, 
-		 conv.xfrLabel(d_algoName);
-		 conv.xfr48BitInt(d_time);
-		 conv.xfr16BitInt(d_fudge);
-		 uint16_t size=d_mac.size();
-		 conv.xfr16BitInt(size);
-		 conv.xfrBlob(d_mac, size);
-		 conv.xfr16BitInt(d_origID);
-		 conv.xfr16BitInt(d_eRcode);
-		 size=d_otherData.size();
-		 conv.xfr16BitInt(size);
-		 conv.xfrBlob(d_otherData, size);
-		 );
+        	 conv.xfrLabel(d_algoName);
+        	 conv.xfr48BitInt(d_time);
+        	 conv.xfr16BitInt(d_fudge);
+        	 uint16_t size=d_mac.size();
+        	 conv.xfr16BitInt(size);
+        	 conv.xfrBlob(d_mac, size);
+        	 conv.xfr16BitInt(d_origID);
+        	 conv.xfr16BitInt(d_eRcode);
+        	 size=d_otherData.size();
+        	 conv.xfr16BitInt(size);
+        	 conv.xfrBlob(d_otherData, size);
+        	 );
 
 MXRecordContent::MXRecordContent(uint16_t preference, const string& mxname) : DNSRecordContent(ns_t_mx), d_preference(preference), d_mxname(mxname)
 {
 }
 
 boilerplate_conv(MX, ns_t_mx, 
-		 conv.xfr16BitInt(d_preference);
-		 conv.xfrLabel(d_mxname, true);
-		 )
+        	 conv.xfr16BitInt(d_preference);
+        	 conv.xfrLabel(d_mxname, true);
+        	 )
 
 boilerplate_conv(KX, ns_t_kx, 
-		 conv.xfr16BitInt(d_preference);
-		 conv.xfrLabel(d_exchanger, false);
-		 )
+        	 conv.xfr16BitInt(d_preference);
+        	 conv.xfrLabel(d_exchanger, false);
+        	 )
 
 boilerplate_conv(IPSECKEY, 45,  /* ns_t_ipsec */
-		 conv.xfr8BitInt(d_preference);
-		 conv.xfr8BitInt(d_gatewaytype);
-		 conv.xfr8BitInt(d_algorithm);
-		 conv.xfrLabel(d_gateway, false);
-		 conv.xfrBlob(d_publickey);
-		 )
+        	 conv.xfr8BitInt(d_preference);
+        	 conv.xfr8BitInt(d_gatewaytype);
+        	 conv.xfr8BitInt(d_algorithm);
+        	 conv.xfrLabel(d_gateway, false);
+        	 conv.xfrBlob(d_publickey);
+        	 )
 
 boilerplate_conv(DHCID, 49, 
-		 conv.xfrBlob(d_content);
-		 )
+        	 conv.xfrBlob(d_content);
+        	 )
 
 
 boilerplate_conv(AFSDB, ns_t_afsdb, 
-		 conv.xfr16BitInt(d_subtype);
-		 conv.xfrLabel(d_hostname);
-		 )
+        	 conv.xfr16BitInt(d_subtype);
+        	 conv.xfrLabel(d_hostname);
+        	 )
 
 
 boilerplate_conv(NAPTR, ns_t_naptr,
-		 conv.xfr16BitInt(d_order);    conv.xfr16BitInt(d_preference);
-		 conv.xfrText(d_flags);        conv.xfrText(d_services);         conv.xfrText(d_regexp);
-		 conv.xfrLabel(d_replacement);
-		 )
+        	 conv.xfr16BitInt(d_order);    conv.xfr16BitInt(d_preference);
+        	 conv.xfrText(d_flags);        conv.xfrText(d_services);         conv.xfrText(d_regexp);
+        	 conv.xfrLabel(d_replacement);
+        	 )
 
 
 SRVRecordContent::SRVRecordContent(uint16_t preference, uint16_t weight, uint16_t port, const string& target) 
@@ -273,9 +187,9 @@ SRVRecordContent::SRVRecordContent(uint16_t preference, uint16_t weight, uint16_
 {}
 
 boilerplate_conv(SRV, ns_t_srv, 
-		 conv.xfr16BitInt(d_preference);   conv.xfr16BitInt(d_weight);   conv.xfr16BitInt(d_port);
-		 conv.xfrLabel(d_target);
-		 )
+        	 conv.xfr16BitInt(d_preference);   conv.xfr16BitInt(d_weight);   conv.xfr16BitInt(d_port);
+        	 conv.xfrLabel(d_target);
+        	 )
 
 
 
@@ -286,94 +200,131 @@ SOARecordContent::SOARecordContent(const string& mname, const string& rname, con
 }
 
 boilerplate_conv(SOA, ns_t_soa, 
-		 conv.xfrLabel(d_mname, true);
-		 conv.xfrLabel(d_rname, true);
-		 conv.xfr32BitInt(d_st.serial);
-		 conv.xfr32BitInt(d_st.refresh);
-		 conv.xfr32BitInt(d_st.retry);
-		 conv.xfr32BitInt(d_st.expire);
-		 conv.xfr32BitInt(d_st.minimum);
-		 );
+        	 conv.xfrLabel(d_mname, true);
+        	 conv.xfrLabel(d_rname, true);
+        	 conv.xfr32BitInt(d_st.serial);
+        	 conv.xfr32BitInt(d_st.refresh);
+        	 conv.xfr32BitInt(d_st.retry);
+        	 conv.xfr32BitInt(d_st.expire);
+        	 conv.xfr32BitInt(d_st.minimum);
+        	 );
 #undef KEY
 boilerplate_conv(KEY, ns_t_key, 
-		 conv.xfr16BitInt(d_flags); 
-		 conv.xfr8BitInt(d_protocol); 
-		 conv.xfr8BitInt(d_algorithm); 
-		 conv.xfrBlob(d_certificate);
-		 );
+        	 conv.xfr16BitInt(d_flags); 
+        	 conv.xfr8BitInt(d_protocol); 
+        	 conv.xfr8BitInt(d_algorithm); 
+        	 conv.xfrBlob(d_certificate);
+        	 );
 
 boilerplate_conv(CERT, 37, 
-		 conv.xfr16BitInt(d_type); 
-		 conv.xfr16BitInt(d_tag); 
-		 conv.xfr8BitInt(d_algorithm); 
-		 conv.xfrBlob(d_certificate);
-		 )
+        	 conv.xfr16BitInt(d_type); 
+        	 conv.xfr16BitInt(d_tag); 
+        	 conv.xfr8BitInt(d_algorithm); 
+        	 conv.xfrBlob(d_certificate);
+        	 )
+		 
+boilerplate_conv(TLSA, 65468, 
+        	 conv.xfr8BitInt(d_certtype); 
+        	 conv.xfr8BitInt(d_hashtype); 
+        	 conv.xfrBlob(d_cert);
+        	 )		 
+		 
 #undef DS
+DSRecordContent::DSRecordContent() : DNSRecordContent(43) {}
 boilerplate_conv(DS, 43, 
-		 conv.xfr16BitInt(d_tag); 
-		 conv.xfr8BitInt(d_algorithm); 
-		 conv.xfr8BitInt(d_digesttype); 
-		 conv.xfrHexBlob(d_digest);
-		 )
+        	 conv.xfr16BitInt(d_tag); 
+        	 conv.xfr8BitInt(d_algorithm); 
+        	 conv.xfr8BitInt(d_digesttype); 
+        	 conv.xfrHexBlob(d_digest, true); // keep reading across spaces
+        	 )
+
+DLVRecordContent::DLVRecordContent() : DNSRecordContent(32769) {}
+boilerplate_conv(DLV,32769 , 
+        	 conv.xfr16BitInt(d_tag); 
+        	 conv.xfr8BitInt(d_algorithm); 
+        	 conv.xfr8BitInt(d_digesttype); 
+        	 conv.xfrHexBlob(d_digest, true); // keep reading across spaces
+        	 )
+
 
 boilerplate_conv(SSHFP, 44, 
-		 conv.xfr8BitInt(d_algorithm); 
-		 conv.xfr8BitInt(d_fptype); 
-		 conv.xfrHexBlob(d_fingerprint);
-		 )
+        	 conv.xfr8BitInt(d_algorithm); 
+        	 conv.xfr8BitInt(d_fptype); 
+        	 conv.xfrHexBlob(d_fingerprint);
+        	 )
 
 boilerplate_conv(RRSIG, 46, 
-		 conv.xfrType(d_type); 
-  		 conv.xfr8BitInt(d_algorithm); 
-  		 conv.xfr8BitInt(d_labels); 
+        	 conv.xfrType(d_type); 
+          	 conv.xfr8BitInt(d_algorithm); 
+          	 conv.xfr8BitInt(d_labels); 
+          	 conv.xfr32BitInt(d_originalttl); 
+          	 conv.xfrTime(d_sigexpire); 
+          	 conv.xfrTime(d_siginception); 
+        	 conv.xfr16BitInt(d_tag); 
+        	 conv.xfrLabel(d_signer);
+        	 conv.xfrBlob(d_signature);
+        	 )
+        	 
+RRSIGRecordContent::RRSIGRecordContent() : DNSRecordContent(46) {}
 
-  		 conv.xfr32BitInt(d_originalttl); 
-  		 conv.xfrTime(d_sigexpire); 
-  		 conv.xfrTime(d_siginception); 
-		 conv.xfr16BitInt(d_tag); 
-		 conv.xfrLabel(d_signer);
-		 conv.xfrBlob(d_signature);
-		 )
-		 
 boilerplate_conv(DNSKEY, 48, 
-		 conv.xfr16BitInt(d_flags); 
-		 conv.xfr8BitInt(d_protocol); 
-		 conv.xfr8BitInt(d_algorithm); 
-		 conv.xfrBlob(d_key);
-		 )
+        	 conv.xfr16BitInt(d_flags); 
+        	 conv.xfr8BitInt(d_protocol); 
+        	 conv.xfr8BitInt(d_algorithm); 
+        	 conv.xfrBlob(d_key);
+        	 )
+DNSKEYRecordContent::DNSKEYRecordContent() : DNSRecordContent(48) {}
+
+uint16_t DNSKEYRecordContent::getTag()
+{
+  string data=this->serialize("");
+  const unsigned char* key=(const unsigned char*)data.c_str();
+  unsigned int keysize=data.length();
+
+  unsigned long ac;     /* assumed to be 32 bits or larger */
+  unsigned int i;                /* loop index */
+  
+  for ( ac = 0, i = 0; i < keysize; ++i )
+    ac += (i & 1) ? key[i] : key[i] << 8;
+  ac += (ac >> 16) & 0xFFFF;
+  return ac & 0xFFFF;
+}
 
 // "fancy records" 
 boilerplate_conv(URL, QType::URL, 
-		 conv.xfrLabel(d_url);
-		 )
+        	 conv.xfrLabel(d_url);
+        	 )
 
 boilerplate_conv(MBOXFW, QType::MBOXFW, 
-		 conv.xfrLabel(d_mboxfw);
-		 )
+        	 conv.xfrLabel(d_mboxfw);
+        	 )
+
+
 
 bool getEDNSOpts(const MOADNSParser& mdp, EDNSOpts* eo)
 {
-  if(mdp.d_header.arcount && !mdp.d_answers.empty() && 
-     mdp.d_answers.back().first.d_type == QType::OPT) {
-    eo->d_packetsize=mdp.d_answers.back().first.d_class;
-    
-    EDNS0Record stuff;
-    uint32_t ttl=ntohl(mdp.d_answers.back().first.d_ttl);
-    memcpy(&stuff, &ttl, sizeof(stuff));
-
-    eo->d_extRCode=stuff.extRCode;
-    eo->d_version=stuff.version;
-    eo->d_Z=stuff.Z;
-    OPTRecordContent* orc = 
-      dynamic_cast<OPTRecordContent*>(mdp.d_answers.back().first.d_content.get());
-    if(!orc)
-      return false;
-    orc->getData(eo->d_options);
-
-    return true;
+  if(mdp.d_header.arcount && !mdp.d_answers.empty()) {
+    BOOST_FOREACH(const MOADNSParser::answers_t::value_type& val, mdp.d_answers) {
+      if(val.first.d_place == DNSRecord::Additional && val.first.d_type == QType::OPT) {
+	eo->d_packetsize=val.first.d_class;
+       
+	EDNS0Record stuff;
+	uint32_t ttl=ntohl(val.first.d_ttl);
+	memcpy(&stuff, &ttl, sizeof(stuff));
+	
+	eo->d_extRCode=stuff.extRCode;
+	eo->d_version=stuff.version;
+	eo->d_Z = ntohs(stuff.Z);
+	OPTRecordContent* orc = 
+	  dynamic_cast<OPTRecordContent*>(val.first.d_content.get());
+	if(!orc)
+	  return false;
+	orc->getData(eo->d_options);
+	return true;
+      }
+    }
   }
-  else
-    return false;
+  return false;
 }
 
 
@@ -389,7 +340,7 @@ void reportBasicTypes()
   PTRRecordContent::report();
   DNSRecordContent::regist(3, ns_t_txt, &TXTRecordContent::make, &TXTRecordContent::make, "TXT");
   TXTRecordContent::report();
-  DNSRecordContent::regist(1, 255, 0, 0, "ANY");
+  DNSRecordContent::regist(1, QType::ANY, 0, 0, "ANY");
 }
 
 void reportOtherTypes()
@@ -407,7 +358,11 @@ void reportOtherTypes()
    SSHFPRecordContent::report();
    CERTRecordContent::report();
    NSECRecordContent::report();
-   TSIGRecordContent::report();
+   NSEC3RecordContent::report();
+   NSEC3PARAMRecordContent::report();
+   TLSARecordContent::report();
+   DLVRecordContent::report();
+   DNSRecordContent::regist(0xff, QType::TSIG, &TSIGRecordContent::make, &TSIGRecordContent::make, "TSIG");
    OPTRecordContent::report();
 }
 
